package com.sun.jersey.samples.ejb;

import com.sun.jersey.api.core.ResourceConfig;
import com.sun.jersey.core.spi.component.ComponentContext;
import com.sun.jersey.core.spi.component.ComponentScope;
import com.sun.jersey.core.spi.component.ioc.IoCComponentProcessorFactory;
import com.sun.jersey.core.spi.component.ioc.IoCComponentProcessorFactoryInitializer;
import com.sun.jersey.core.spi.component.ioc.IoCComponentProvider;
import com.sun.jersey.core.spi.component.ioc.IoCComponentProviderFactory;
import com.sun.jersey.core.spi.component.ioc.IoCFullyManagedComponentProvider;
import com.sun.jersey.spi.container.WebApplication;
import com.sun.jersey.spi.container.servlet.ServletContainer;
import javax.annotation.PostConstruct;
import javax.ejb.EJBException;
import javax.ejb.Stateless;
import javax.interceptor.InvocationContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class EJBIntegrationServlet extends ServletContainer {


    public static class InjectionInterceptor {

        // This is a hack
        // An instance InjectionInterceptor configured with
        // IoCComponentProcessorFactory
        private static IoCComponentProcessorFactory cpf;

        @PostConstruct
        private void init(InvocationContext context) throws Exception {

            Object beanInstance = context.getTarget();

            cpf.get(beanInstance.getClass(), ComponentScope.Singleton).
                    postConstruct(beanInstance);

            // Invoke next interceptor in chain
            context.proceed();
        }
    }

    private static class EJBComponentProvider implements 
            IoCComponentProviderFactory,
            IoCComponentProcessorFactoryInitializer {

        // IoCComponentProviderFactory

        public IoCComponentProvider getComponentProvider(Class<?> c) {
            return getComponentProvider(null, c);
        }

        public IoCComponentProvider getComponentProvider(ComponentContext cc, Class<?> c) {
            if (c.isAnnotationPresent(Stateless.class)) {
                try {
                    InitialContext ic = new InitialContext();

                    Object o = ic.lookup(getName(c));

                    return new EJBManagedComponentProvider(o);
                } catch(NamingException ne) {
                    throw new EJBException(ne);
                }                
            } else {
                return null;
            }
        }

        // IoCComponentProcessorFactoryInitializer
        
        public void init(IoCComponentProcessorFactory cpf) {
            InjectionInterceptor.cpf = cpf;
        }

        private String getName(Class<?> c) {
            Stateless s = c.getAnnotation(Stateless.class);

            String simpleName = s.name();
            if (simpleName == null || simpleName.length() == 0)
                simpleName = c.getSimpleName();

            return "java:module/" + simpleName + "!" + c.getName();
        }

    }

    private static class EJBManagedComponentProvider implements IoCFullyManagedComponentProvider {
        private final Object o;

        EJBManagedComponentProvider(Object o) {
            this.o = o;
        }

        public Object getInstance() {
            return o;
        }
    }

    @Override
    protected void initiate(ResourceConfig rc, WebApplication wa) {
        wa.initiate(rc, new EJBComponentProvider());
    }
}
